@@ -8,31 +8,11 @@
 // Description :
 //
 //-----------------------------------------------------------------------------
-// LICENSE
-// (c) 2024, Steinberg Media Technologies GmbH, All Rights Reserved
-//-----------------------------------------------------------------------------
-// Redistribution and use in source and binary forms, with or without modification,
-// are permitted provided that the following conditions are met:
-//
-//   * Redistributions of source code must retain the above copyright notice,
-//     this list of conditions and the following disclaimer.
-//   * Redistributions in binary form must reproduce the above copyright notice,
-//     this list of conditions and the following disclaimer in the documentation
-//     and/or other materials provided with the distribution.
-//   * Neither the name of the Steinberg Media Technologies nor the names of its
-//     contributors may be used to endorse or promote products derived from this
-//     software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-// IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
-// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
-// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
-// OF THE POSSIBILITY OF SUCH DAMAGE.
+// This file is part of a Steinberg SDK. It is subject to the license terms
+// in the LICENSE file found in the top-level directory of this distribution
+// and at www.steinberg.net/sdklicenses.
+// No part of the SDK, including this file, may be copied, modified, propagated,
+// or distributed except according to the terms contained in the LICENSE file.
 //-----------------------------------------------------------------------------
 
 #include "hostcheckerprocessor.h"
@@ -197,12 +177,11 @@ void HostCheckerProcessor::sendLogEventMessage (const LogEvent& logEvent)
 	if (auto message = owned (allocateMessage ()))
 	{
 		message->setMessageID ("LogEvent");
-		IAttributeList* attributes = message->getAttributes ();
-		if (attributes)
+		if (auto* attr = message->getAttributes ())
 		{
 			SMTG_ASSERT (logEvent.id >= 0);
-			attributes->setInt ("ID", logEvent.id);
-			attributes->setInt ("Count", logEvent.count);
+			attr->setInt ("ID", logEvent.id);
+			attr->setInt ("Count", logEvent.count);
 			sendMessage (message);
 		}
 	}
@@ -274,12 +253,14 @@ void HostCheckerProcessor::informLatencyChanged ()
 //-----------------------------------------------------------------------------
 void HostCheckerProcessor::sendLatencyChanged ()
 {
-	if (IMessage* newMsg = allocateMessage ())
+	if (auto message = owned (allocateMessage ()))
 	{
-		newMsg->setMessageID ("Latency");
-		if (auto* attr = newMsg->getAttributes ())
-			attr->setFloat ("Value", mWantedLatency);
-		sendMessage (newMsg);
+		message->setMessageID ("Latency");
+		if (auto* attr = message->getAttributes ())
+		{
+			attr->setInt ("Value", mWantedLatency);
+		}
+		sendMessage (message);
 	}
 }
 
@@ -323,6 +304,7 @@ tresult PLUGIN_API HostCheckerProcessor::process (ProcessData& data)
 	else if (data.processMode == kPrefetch)
 		addLogEvent (kLogIdProcessModePrefetchSupported);
 
+	bool changeFromPlayToStop = false;
 	if (data.processContext)
 	{
 		if (dataExchangeHandler)
@@ -377,6 +359,8 @@ tresult PLUGIN_API HostCheckerProcessor::process (ProcessData& data)
 		{
 			bool playbackChanged = (data.processContext->state & ProcessContext::kPlaying) !=
 			                       (mLastState & ProcessContext::kPlaying);
+			changeFromPlayToStop = (mLastState & ProcessContext::kPlaying) &&
+			                       !(data.processContext->state & ProcessContext::kPlaying);
 			if ((mLastState & ProcessContext::kPlaying) == 0)
 			{
 				if (mLastProjectTimeSamples != data.processContext->projectTimeSamples)
@@ -427,7 +411,8 @@ tresult PLUGIN_API HostCheckerProcessor::process (ProcessData& data)
 			}
 			else if (id == kLatencyTag)
 			{
-				mWantedLatency = static_cast<uint32> (value * HostChecker::kMaxLatency);
+				mWantedLatency = static_cast<uint32> (value * HostChecker::kMaxLatencyInSeconds *
+				                                      data.processContext->sampleRate);
 				addLogEvent (kLogIdInformLatencyChanged);
 			}
 			else if (id == kProcessingLoadTag)
@@ -443,6 +428,13 @@ tresult PLUGIN_API HostCheckerProcessor::process (ProcessData& data)
 
 	if (mBypassProcessorFloat.isActive ())
 	{
+		if (changeFromPlayToStop)
+		{
+			mBypassProcessorFloat.setActive (false);
+			mBypassProcessorDouble.setActive (false);
+			mBypassProcessorFloat.setActive (true);
+			mBypassProcessorDouble.setActive (true);
+		}
 		if (data.symbolicSampleSize == kSample32)
 			mBypassProcessorFloat.process (data);
 		else // kSample64
