@@ -7,31 +7,11 @@
 // Description : VST 3 AUv3Wrapper
 //
 //-----------------------------------------------------------------------------
-// LICENSE
-// (c) 2023, Steinberg Media Technologies GmbH, All Rights Reserved
-//-----------------------------------------------------------------------------
-// Redistribution and use in source and binary forms, with or without modification,
-// are permitted provided that the following conditions are met:
-//
-//   * Redistributions of source code must retain the above copyright notice,
-//     this list of conditions and the following disclaimer.
-//   * Redistributions in binary form must reproduce the above copyright notice,
-//     this list of conditions and the following disclaimer in the documentation
-//     and/or other materials provided with the distribution.
-//   * Neither the name of the Steinberg Media Technologies nor the names of its
-//     contributors may be used to endorse or promote products derived from this
-//     software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
-// IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
-// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
-// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
-// OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  OF THIS SOFTWARE, EVEN IF ADVISED
-// OF THE POSSIBILITY OF SUCH DAMAGE.
+// This file is part of a Steinberg SDK. It is subject to the license terms
+// in the LICENSE file found in the top-level directory of this distribution
+// and at www.steinberg.net/sdklicenses.
+// No part of the SDK, including this file, may be copied, modified, propagated,
+// or distributed except according to the terms contained in the LICENSE file.
 //-----------------------------------------------------------------------------
 
 #import "AUv3Wrapper.h"
@@ -44,6 +24,7 @@
 #import "public.sdk/source/vst/hosting/processdata.h"
 #import "public.sdk/source/vst/utility/mpeprocessor.h"
 #import "public.sdk/source/vst/utility/stringconvert.h"
+#import "pluginterfaces/base/funknownimpl.h"
 #import "pluginterfaces/gui/iplugview.h"
 #import "pluginterfaces/vst/ivsteditcontroller.h"
 #import "pluginterfaces/vst/ivstmidicontrollers.h"
@@ -62,9 +43,11 @@
 #if TARGET_OS_IPHONE
 #define SMTG_IOS_MAC_PLATFORMTYPE kPlatformTypeUIView
 #define SMTG_IOS_MAC_VIEW UIView
+#define SMTG_IOS_MAC_AutoresizingMask_NotSizable UIViewAutoresizingNone
 #else
 #define SMTG_IOS_MAC_VIEW NSView
 #define SMTG_IOS_MAC_PLATFORMTYPE kPlatformTypeNSView
+#define SMTG_IOS_MAC_AutoresizingMask_NotSizable NSViewNotSizable
 #endif
 
 extern "C" {
@@ -160,6 +143,18 @@ Steinberg::IPluginFactory* PLUGIN_API GetPluginFactory ()
 //------------------------------------------------------------------------
 namespace Steinberg {
 
+//------------------------------------------------------------------------
+template<typename Proc>
+bool executeOnMainThreadSync (Proc p)
+{
+	if (pthread_main_np () == 0)
+	{
+		dispatch_sync (dispatch_get_main_queue (), p);
+		return true;
+	}
+	return false;
+}
+
 //------------------------------------------------------------------------
 class AUPlugFrame final : public IPlugFrame
 {
@@ -235,7 +230,7 @@ public:
 
 	tresult PLUGIN_API getName (String128 name) SMTG_OVERRIDE
 	{
-		return VST3::StringConvert::convert ("VST3-AU Wrapper", name) ? kResultTrue : kResultFalse;
+		return Vst::StringConvert::convert ("VST3-AU Wrapper", name) ? kResultTrue : kResultFalse;
 	}
 
 	tresult PLUGIN_API enableMPEInputProcessing (TBool state) SMTG_OVERRIDE
@@ -425,6 +420,7 @@ IMPLEMENT_QUERYINTERFACE (ComponentHelper, IComponentHandler, IComponentHandler:
 //------------------------------------------------------------------------
 tresult PLUGIN_API ComponentHelper::beginEdit (ParamID tag)
 {
+	[auv3Wrapper beginEdit:tag];
 	return kResultTrue;
 }
 
@@ -438,6 +434,7 @@ tresult PLUGIN_API ComponentHelper::performEdit (ParamID tag, ParamValue valueNo
 //------------------------------------------------------------------------
 tresult PLUGIN_API ComponentHelper::endEdit (ParamID tag)
 {
+	[auv3Wrapper endEdit:tag];
 	return kResultTrue;
 }
 
@@ -604,7 +601,7 @@ private:
 	{
 		if (!context->inputEvents)
 			return;
-		Event e = {0};
+		Event e {};
 		e.type = Event::kPolyPressureEvent;
 		e.polyPressure.channel = channel;
 		e.polyPressure.pitch = pitch;
@@ -979,8 +976,7 @@ using namespace Vst;
 	if (!self.supportsMPE)
 		return;
 
-	FUnknownPtr<INoteExpressionPhysicalUIMapping> puiMapping (_editcontroller);
-	if (puiMapping)
+	if (auto puiMapping = U::cast<INoteExpressionPhysicalUIMapping> (_editcontroller))
 	{
 		PhysicalUIMap map[3];
 		map[0].physicalUITypeID = kPUIXMovement;
@@ -997,8 +993,7 @@ using namespace Vst;
 			mpeHandlerContext.noteExpValueDescMap[0] = {};
 			mpeHandlerContext.noteExpValueDescMap[1] = {};
 			mpeHandlerContext.noteExpValueDescMap[2] = {};
-			FUnknownPtr<INoteExpressionController> nExpCtrler (_editcontroller);
-			if (nExpCtrler)
+			if (auto nExpCtrler = U::cast<INoteExpressionController> (_editcontroller))
 			{
 				auto count = nExpCtrler->getNoteExpressionCount (0, 0);
 				for (auto i = 0; i < count; ++i)
@@ -1049,11 +1044,11 @@ using namespace Vst;
 {
 	if (audioProcessor)
 	{
-		FUnknownPtr<IEditController> combined (audioProcessor);
+		auto combined = U::cast<IEditController> (audioProcessor);
 		if (!combined)
 		{
-			FUnknownPtr<IConnectionPoint> controllerConnectionPoint (_editcontroller);
-			FUnknownPtr<IConnectionPoint> processorConnectionPoint (audioProcessor);
+			auto controllerConnectionPoint = U::cast<IConnectionPoint> (_editcontroller);
+			auto processorConnectionPoint = U::cast<IConnectionPoint> (audioProcessor);
 			if (controllerConnectionPoint && processorConnectionPoint)
 			{
 				controllerConnectionPoint->disconnect (processorConnectionPoint);
@@ -1133,33 +1128,33 @@ using namespace Vst;
 //------------------------------------------------------------------------
 - (void)loadVst
 {
-	FUnknownPtr<IPluginFactory2> factory (owned (GetPluginFactory ()));
-	if (factory)
+	auto factory = U::cast<IPluginFactory2> (owned (GetPluginFactory ()));
+	if (!factory)
+		return;
+
+	// find first audio processor class
+	for (int32 i = 0; i < factory->countClasses (); i++)
 	{
-		// find first audio processor class
-		for (int32 i = 0; i < factory->countClasses (); i++)
+		PClassInfo2 ci;
+		if (factory->getClassInfo2 (i, &ci) == kResultTrue)
 		{
-			PClassInfo2 ci;
-			if (factory->getClassInfo2 (i, &ci) == kResultTrue)
+			if (strcmp (ci.category, kVstAudioEffectClass) == 0)
 			{
-				if (strcmp (ci.category, kVstAudioEffectClass) == 0)
+				IAudioProcessor* proc = nullptr;
+				if (factory->createInstance (ci.cid, IAudioProcessor::iid, (void**)&proc) ==
+					kResultTrue)
 				{
-					IAudioProcessor* proc = nullptr;
-					if (factory->createInstance (ci.cid, IAudioProcessor::iid, (void**)&proc) ==
-					    kResultTrue)
-					{
-						audioProcessor = owned (proc);
-						std::string_view plugCategory (ci.subCategories);
-						std::string_view instrumentStr ("instrument");
-						auto it = std::search (plugCategory.begin (), plugCategory.end (),
-						                       instrumentStr.begin (), instrumentStr.end (),
-						                       [] (auto ch1, auto ch2) {
-							                       return std::toupper (ch1) == std::toupper (ch2);
-						                       });
-						if (it != plugCategory.end ())
-							isInstrument = true;
-						break;
-					}
+					audioProcessor = owned (proc);
+					std::string_view plugCategory (ci.subCategories);
+					std::string_view instrumentStr ("instrument");
+					auto it = std::search (plugCategory.begin (), plugCategory.end (),
+						                    instrumentStr.begin (), instrumentStr.end (),
+						                    [] (auto ch1, auto ch2) {
+							                    return std::toupper (ch1) == std::toupper (ch2);
+						                    });
+					if (it != plugCategory.end ())
+						isInstrument = true;
+					break;
 				}
 			}
 		}
@@ -1174,8 +1169,7 @@ using namespace Vst;
 		if (audioProcessor->queryInterface (IEditController::iid, (void**)&_editcontroller) !=
 		    kResultTrue)
 		{
-			FUnknownPtr<IComponent> component (audioProcessor);
-			if (component)
+			if (auto component = U::cast<IComponent> (audioProcessor))
 			{
 				TUID ccid;
 				if (component->getControllerClassId (ccid) == kResultTrue)
@@ -1191,8 +1185,8 @@ using namespace Vst;
 							return;
 						}
 
-						FUnknownPtr<IConnectionPoint> controllerConnectionPoint (_editcontroller);
-						FUnknownPtr<IConnectionPoint> processorConnectionPoint (audioProcessor);
+						auto controllerConnectionPoint = U::cast<IConnectionPoint> (_editcontroller);
+						auto processorConnectionPoint = U::cast<IConnectionPoint> (audioProcessor);
 						if (controllerConnectionPoint && processorConnectionPoint)
 						{
 							controllerConnectionPoint->connect (processorConnectionPoint);
@@ -1224,7 +1218,7 @@ using namespace Vst;
 	{
 		_editcontroller->setComponentHandler (&componentHelper);
 
-		FUnknownPtr<IComponent> component (audioProcessor);
+		auto component = U::cast<IComponent> (audioProcessor);
 		int32 inputBusCount = component->getBusCount (kAudio, kInput);
 		int32 outputBusCount = component->getBusCount (kAudio, kOutput);
 
@@ -1256,7 +1250,7 @@ using namespace Vst;
 - (void)createBusses:(bool)isInput
             busCount:(int)busCount
             busArray:(NSMutableArray<AUAudioUnitBus*>*)busArray
-           component:(FUnknownPtr<IComponent>)component
+           component:(IPtr<IComponent>)component
 {
 	SpeakerArrangement sa;
 	int inputOrOutput = isInput ? kInput : kOutput;
@@ -1276,7 +1270,7 @@ using namespace Vst;
 			bus.maximumChannelCount = customFormat.channelCount;
 		}
 
-		BusInfo info = {0};
+		BusInfo info {};
 		if (component->getBusInfo (kAudio, inputOrOutput, busNum, info) == kResultTrue)
 		{
 			bus.name = createCFStringFromString128 (info.name);
@@ -1292,7 +1286,7 @@ using namespace Vst;
 //------------------------------------------------------------------------
 - (void)allocateBusBuffers
 {
-	FUnknownPtr<IComponent> component (audioProcessor);
+	auto component = U::cast<IComponent> (audioProcessor);
 
 	for (int i = 0; i < component->getBusCount (kAudio, kInput); i++)
 	{
@@ -1418,7 +1412,7 @@ using namespace Vst;
 	int32 parameterCount = _editcontroller->getParameterCount ();
 	for (int32 i = 0; i < parameterCount; i++)
 	{
-		ParameterInfo pi = {0};
+		ParameterInfo pi {};
 		_editcontroller->getParameterInfo (i, pi);
 
 		if (pi.flags & ParameterInfo::kIsBypass)
@@ -1490,8 +1484,7 @@ using namespace Vst;
 {
 	int32 groupIdx = 0; // Zero means parameter is on top level node of parameter tree
 
-	FUnknownPtr<IUnitInfo> unitInfoController (_editcontroller);
-	if (unitInfoController)
+	if (auto unitInfoController = U::cast<IUnitInfo> (_editcontroller))
 	{
 		if (unitInfos.empty ())
 			buildUnitInfos (unitInfoController, unitInfos);
@@ -1541,7 +1534,7 @@ using namespace Vst;
 {
 	if (audioProcessor && _editcontroller)
 	{
-		FUnknownPtr<IComponent> component (audioProcessor);
+		auto component = U::cast<IComponent> (audioProcessor);
 
 		// after set Bus Arrangement, the channelbuffers may need to be reallocated -> hence the
 		// second prepare!
@@ -1580,7 +1573,7 @@ using namespace Vst;
 			int32 unitCount = unitInfo->getUnitCount ();
 			for (int32 i = 0; i < unitCount; i++)
 			{
-				UnitInfo unitInfoStruct = {0};
+				UnitInfo unitInfoStruct {};
 				if (unitInfo->getUnitInfo (i, unitInfoStruct) == kResultTrue)
 				{
 					if (unitId == unitInfoStruct.id)
@@ -1608,7 +1601,7 @@ using namespace Vst;
 	int32 parameterCount = _editcontroller->getParameterCount ();
 	for (int32 i = 0; i < parameterCount; i++)
 	{
-		ParameterInfo pi = {0};
+		ParameterInfo pi {};
 		_editcontroller->getParameterInfo (i, pi);
 
 		// do not register bypass
@@ -1648,7 +1641,7 @@ using namespace Vst;
 		}
 	}
 
-	FUnknownPtr<IUnitInfo> unitInfoController (_editcontroller);
+	auto unitInfoController = U::cast<IUnitInfo> (_editcontroller);
 
 	factoryPresetsVar = [[NSMutableArray<AUAudioUnitPreset*> alloc] init];
 
@@ -1674,7 +1667,7 @@ using namespace Vst;
 			int32 unitCount = unitInfo->getUnitCount ();
 			for (int32 i = 0; i < unitCount; i++)
 			{
-				UnitInfo unitInfoStruct = {0};
+				UnitInfo unitInfoStruct {};
 				if (unitInfo->getUnitInfo (i, unitInfoStruct) == kResultTrue)
 				{
 					if (programListInfo.id == unitInfoStruct.programListId)
@@ -1771,14 +1764,33 @@ using namespace Vst;
 }
 
 //------------------------------------------------------------------------
-- (void)performEdit:(int)tag value:(double)value
+- (void)beginEdit:(int32_t)tag
 {
 	AUParameter* parameterToChange = [parameterTreeVar parameterWithAddress:tag];
+	[parameterToChange setValue:parameterToChange.value
+	                 originator:parameterObserverToken
+	                 atHostTime:0
+	                  eventType:AUParameterAutomationEventTypeTouch];
+}
 
-	if (parameterObserverToken != nullptr)
-		[parameterToChange setValue:value originator:parameterObserverToken];
-	else
-		[parameterToChange setValue:value];
+//------------------------------------------------------------------------
+- (void)endEdit:(int32_t)tag
+{
+	AUParameter* parameterToChange = [parameterTreeVar parameterWithAddress:tag];
+	[parameterToChange setValue:parameterToChange.value
+	                 originator:parameterObserverToken
+	                 atHostTime:0
+	                  eventType:AUParameterAutomationEventTypeRelease];
+}
+
+//------------------------------------------------------------------------
+- (void)performEdit:(int32_t)tag value:(double)value
+{
+	AUParameter* parameterToChange = [parameterTreeVar parameterWithAddress:tag];
+	[parameterToChange setValue:value
+	                 originator:parameterObserverToken
+	                 atHostTime:0
+	                  eventType:AUParameterAutomationEventTypeValue];
 }
 
 //------------------------------------------------------------------------
@@ -1814,7 +1826,7 @@ using namespace Vst;
 //------------------------------------------------------------------------
 - (BOOL)validateChannelConfig
 {
-	FUnknownPtr<IComponent> component (audioProcessor);
+	auto component = U::cast<IComponent> (audioProcessor);
 	int32 inputBusCount = component->getBusCount (kAudio, kInput);
 	int32 outputBusCount = component->getBusCount (kAudio, kOutput);
 	SpeakerArrangement inputs[inputBusCount];
@@ -2028,8 +2040,7 @@ using namespace Vst;
 {
 	if (_editcontroller)
 	{
-		FUnknownPtr<INoteExpressionController> nec (_editcontroller);
-		if (nec)
+		if (auto nec = U::cast<INoteExpressionController> (_editcontroller))
 			return YES;
 	}
 	return NO;
@@ -2097,6 +2108,9 @@ using namespace Vst;
 	if (nil == presentPreset)
 		return;
 
+	if (executeOnMainThreadSync (^{ [self setCurrentPreset:presentPreset]; }))
+		return;
+
 	// if it is a factory preset
 	if (presentPreset.number >= 0)
 	{
@@ -2131,12 +2145,15 @@ using namespace Vst;
 {
 	// should flush parameters of VST to save correct state (when not in playback mode)
 
-	NSMutableDictionary<NSString*, id>* dict = [[NSMutableDictionary<NSString*, id> alloc] init];
+	__block NSDictionary<NSString*, id>* mtResult = nil;
+	if (executeOnMainThreadSync (^{ mtResult = [self fullState]; }))
+	{
+		return mtResult;
+	}
 
-	NSDictionary<NSString*, id>* superDict = [super fullState];
-
-	if (superDict != nullptr)
-		[dict addEntriesFromDictionary:superDict];
+	auto* result = [NSMutableDictionary<NSString*, id> new];
+	if (auto* superDict = [super fullState])
+		[result addEntriesFromDictionary:superDict];
 
 	NSMutableData* processorData = [[NSMutableData alloc] init];
 	NSMutableData* controllerData = [[NSMutableData alloc] init];
@@ -2157,19 +2174,21 @@ using namespace Vst;
 			[controllerData setLength:0];
 	}
 
-	[dict setObject:processorData forKey:@"Processor State"];
-	[dict setObject:controllerData forKey:@"Controller State"];
+	[result setObject:processorData forKey:@"Processor State"];
+	[result setObject:controllerData forKey:@"Controller State"];
 
-	return dict;
+	return result;
 }
 
 //------------------------------------------------------------------------
 - (void)setFullState:(NSDictionary<NSString*, id>*)newFullState
 {
-
 	if (newFullState == nullptr)
 		return;
 
+	if (executeOnMainThreadSync (^{ [self setFullState:newFullState]; }))
+		return;
+
 	bool fromProject = false;
 
 	fullStateVar = newFullState;
@@ -2196,14 +2215,14 @@ using namespace Vst;
 
 	std::u16string projectStr;
 	if (fromProject)
-		projectStr = VST3::StringConvert::convert (Vst::StateType::kProject);
+		projectStr = Vst::StringConvert::convert (Vst::StateType::kProject);
 	if (processorData)
 	{
 		NSDataIBStream stream (processorData);
 		if (fromProject)
 		{
 			stream.getAttributes ()->setString (Vst::PresetAttributes::kStateType,
-			                                    VST3::toTChar (projectStr));
+			                                    Vst::toTChar (projectStr));
 		}
 		FUnknownPtr<IComponent> (audioProcessor)->setState (&stream);
 	}
@@ -2219,7 +2238,7 @@ using namespace Vst;
 		if (fromProject)
 		{
 			stream.getAttributes ()->setString (Vst::PresetAttributes::kStateType,
-			                                    VST3::toTChar (projectStr));
+			                                    Vst::toTChar (projectStr));
 		}
 		_editcontroller->setState (&stream);
 
@@ -2234,8 +2253,18 @@ using namespace Vst;
 }
 
 //------------------------------------------------------------------------
-- (BOOL)allocateRenderResourcesAndReturnError:(NSError**)outError
+- (BOOL)allocateRenderResourcesAndReturnError:(NSError* __autoreleasing*)outError
 {
+	__block BOOL result = NO;
+	__block NSError* errorString = nullptr;
+	if (executeOnMainThreadSync (
+	        ^{ result = [self allocateRenderResourcesAndReturnError:&errorString]; }))
+	{
+		if (outError)
+			*outError = errorString;
+		return result;
+	}
+
 	if (![super allocateRenderResourcesAndReturnError:outError])
 		return NO;
 
@@ -2274,6 +2303,9 @@ using namespace Vst;
 //------------------------------------------------------------------------
 - (void)deallocateRenderResources
 {
+	if (executeOnMainThreadSync (^{ [self deallocateRenderResources]; }))
+		return;
+
 	musicalContext = nullptr;
 	transportContext = nullptr;
 
@@ -2290,8 +2322,8 @@ using namespace Vst;
 	{
 		audioProcessor->setProcessing (false);
 
-		FUnknownPtr<IComponent> component (audioProcessor);
-		component->setActive (false);
+		if (auto component = U::cast<IComponent> (audioProcessor))
+			component->setActive (false);
 	}
 
 	[super deallocateRenderResources];
@@ -2347,7 +2379,7 @@ using namespace Vst;
 
 	  if (frameCount > 0)
 	  {
-		  FUnknownPtr<IComponent> component (audioProcessor);
+		  auto component = U::cast<IComponent> (audioProcessor);
 		  int32 inputBusCount = component->getBusCount (kAudio, kInput);
 		  int32 outputBusCount = component->getBusCount (kAudio, kOutput);
 		  AudioUnitRenderActionFlags pullFlags = 0;
@@ -2465,11 +2497,11 @@ using namespace Vst;
 - (void)loadView
 {
 	SMTG_IOS_MAC_VIEW* view = [[SMTG_IOS_MAC_VIEW alloc] initWithFrame:CGRectMake (0, 0, 0, 0)];
-	view.autoresizingMask = NSViewNotSizable;
+	view.autoresizingMask = SMTG_IOS_MAC_AutoresizingMask_NotSizable;
 	view.translatesAutoresizingMaskIntoConstraints = YES;
 	[self setView:view];
 
-#if !TARGET_API_IPHONE
+#if SMTG_OS_OSX
 	//-------------------------------------------
 	// workaround bug of Logic/Garageband to not listen to plug-in editor size changes and using the
 	// initial size and loading the view before creating the audio unit. (FB8971597)
@@ -2506,7 +2538,7 @@ using namespace Vst;
 //------------------------------------------------------------------------
 - (void)setFrame:(CGRect)newSize
 {
-	if (NSEqualRects (self.view.frame, newSize))
+	if (CGRectEqualToRect (self.view.frame, newSize))
 		return;
 
 	self.view.frame = newSize;
@@ -2529,8 +2561,7 @@ using namespace Vst;
 - (void)makePlugView
 {
 	assert (pthread_main_np () != 0);
-	FUnknownPtr<IEditController2> ec2 (editController);
-	if (ec2)
+	if (auto ec2 = U::cast<IEditController2> (editController))
 		ec2->setKnobMode (kLinearMode);
 
 	// create view
