@@ -179,6 +179,8 @@ struct CView::Impl
 	using ViewListenerDispatcher = DispatchList<IViewListener*>;
 	using ViewEventListenerDispatcher = DispatchList<IViewEventListener*>;
 
+	static uint64_t gRuntimeID;
+
 	ViewAttributes attributes;
 	std::unique_ptr<ViewListenerDispatcher> viewListeners;
 	std::unique_ptr<ViewEventListenerDispatcher> viewEventListeners;
@@ -193,8 +195,12 @@ struct CView::Impl
 	int32_t autosizeFlags {kAutosizeNone};
 	CFrame* parentFrame {nullptr};
 	CView* parentView {nullptr};
+	uint64_t runtimeID {++gRuntimeID};
 };
 
+//-----------------------------------------------------------------------------
+uint64_t CView::Impl::gRuntimeID = 0u;
+
 //-----------------------------------------------------------------------------
 CView::CView (const CRect& size)
 {
@@ -272,6 +278,12 @@ void CView::beforeDelete ()
 #endif
 }
 
+//------------------------------------------------------------------------
+uint64_t CView::getRuntimeID () const
+{
+	return pImpl->runtimeID;
+}
+
 //-----------------------------------------------------------------------------
 void CView::setMouseableArea (const CRect& rect)
 {
