@@ -4,7 +4,6 @@
 
 #include "win32window.h"
 #include "win32menu.h"
-#include "win32async.h"
 
 #include "../../../../lib/platform/win32/win32directcomposition.h"
 #include "../../../../lib/platform/win32/win32factory.h"
@@ -13,6 +12,7 @@
 #include "../../../../lib/platform/win32/winstring.h"
 #include "../../../include/iappdelegate.h"
 #include "../../../include/iapplication.h"
+#include "../../../include/iasync.h"
 #include "../../application.h"
 #include "../iplatformwindow.h"
 #include <dwmapi.h>
@@ -505,11 +505,22 @@ LRESULT CALLBACK Window::proc (UINT message, WPARAM wParam, LPARAM lParam)
 				frame->getTransform ().inverse ().transform (p);
 				p = delegate->constraintSize (p);
 				frame->getTransform ().transform (p);
-				minmaxInfo->ptMinTrackSize = mapCPoint (p);
+				auto point = mapCPoint (p);
+				RECT clientRect {0, 0, point.x, point.y};
+				HiDPISupport::instance ().adjustWindowRectExForDpi (
+					&clientRect, dwStyle, !menuCommandList.empty (), exStyle,
+					static_cast<UINT> (dpiScale * USER_DEFAULT_SCREEN_DPI));
+				minmaxInfo->ptMinTrackSize = {clientRect.right - clientRect.left,
+											  clientRect.bottom - clientRect.top};
 				p = mapPOINT (minmaxInfo->ptMaxTrackSize);
 				frame->getTransform ().inverse ().transform (p);
 				p = delegate->constraintSize (p);
 				frame->getTransform ().transform (p);
+				point = mapCPoint (p);
+				clientRect = {0, 0, point.x, point.y};
+				HiDPISupport::instance ().adjustWindowRectExForDpi (
+					&clientRect, dwStyle, !menuCommandList.empty (), exStyle,
+					static_cast<UINT> (dpiScale * USER_DEFAULT_SCREEN_DPI));
 				minmaxInfo->ptMaxTrackSize = mapCPoint (p);
 				return 0;
 			}
@@ -677,9 +688,9 @@ LRESULT CALLBACK Window::proc (UINT message, WPARAM wParam, LPARAM lParam)
 			RECT clientRect {};
 			clientRect.right = static_cast<LONG> (clientSize.x * newScaleFactor);
 			clientRect.bottom = static_cast<LONG> (clientSize.y * newScaleFactor);
-			HiDPISupport::instance ().adjustWindowRectExForDpi (&clientRect, windowInfo.dwStyle,
-																hasMenu, windowInfo.dwExStyle,
-																static_cast<UINT> (wParam));
+			HiDPISupport::instance ().adjustWindowRectExForDpi (
+				&clientRect, windowInfo.dwStyle, !menuCommandList.empty (), windowInfo.dwExStyle,
+				static_cast<UINT> (wParam));
 			proposedSize->cx = clientRect.right - clientRect.left;
 			proposedSize->cy = clientRect.bottom - clientRect.top;
 			return TRUE;
@@ -897,7 +908,7 @@ void Window::setSize (const CPoint& newSize)
 	clientRect.right = static_cast<LONG> (newSize.x * dpiScale);
 	clientRect.bottom = static_cast<LONG> (newSize.y * dpiScale);
 	HiDPISupport::instance ().adjustWindowRectExForDpi (
-		&clientRect, dwStyle, hasMenu, exStyle,
+		&clientRect, dwStyle, !menuCommandList.empty (), exStyle,
 		static_cast<UINT> (dpiScale * USER_DEFAULT_SCREEN_DPI));
 
 	LONG width = clientRect.right - clientRect.left;
@@ -919,7 +930,7 @@ void Window::setPosition (const CPoint& newPosition)
 	clientRect.right = 100;
 	clientRect.bottom = 100;
 	HiDPISupport::instance ().adjustWindowRectExForDpi (
-		&clientRect, dwStyle, hasMenu, exStyle,
+		&clientRect, dwStyle, !menuCommandList.empty (), exStyle,
 		static_cast<UINT> (dpiScale * USER_DEFAULT_SCREEN_DPI));
 
 	clientRect.left += static_cast<LONG> (newPosition.x);
@@ -989,7 +1000,7 @@ void Window::show ()
 	RECT clientRect {};
 	clientRect.right = static_cast<LONG> (initialSize.x * dpiScale);
 	clientRect.bottom = static_cast<LONG> (initialSize.y * dpiScale);
-	AdjustWindowRectEx (&clientRect, dwStyle, hasMenu, exStyle);
+	AdjustWindowRectEx (&clientRect, dwStyle, !menuCommandList.empty (), exStyle);
 
 	delegate->onShow ();
 	LONG width = clientRect.right - clientRect.left;
