@@ -42,7 +42,9 @@ using UIDesc::ModelBindingPtr;
 using UIDesc::CustomizationPtr;
 
 //------------------------------------------------------------------------
-class WindowController : public IWindowController, public ICommandHandler
+class WindowController : public IWindowController,
+						 public ICommandHandler,
+						 public IUIDescWindowController
 {
 public:
 	bool init (const UIDesc::Config& config, WindowPtr& window);
@@ -66,6 +68,9 @@ public:
 
 	const IMenuBuilder* getWindowMenuBuilder (const IWindow& window) const override;
 
+	ModelBindingPtr getModelBinding () const override;
+	CustomizationPtr getCustomization () const override;
+
 private:
 	struct Impl;
 	struct EditImpl;
@@ -498,6 +503,7 @@ struct WindowController::Impl : public IController, public ICommandHandler
 
 	void showView ()
 	{
+		frame->removeAll ();
 		updateMinMaxSizes ();
 		auto view = uiDesc->createView (templateName, this);
 		if (!view)
@@ -679,6 +685,7 @@ struct WindowController::EditImpl : WindowController::Impl
 	: Impl (controller, modelBinding, customization)
 	{
 		IApplication::instance ().registerCommand (Commands::Debug::ToggleInlineUIEditor, 'E');
+		IApplication::instance ().registerCommand (Commands::Debug::RecreateView, 0);
 		if (IApplication::instance ().getDelegate ().getSharedUIResourceFilename ())
 			IApplication::instance ().registerCommand (Commands::Debug::ResaveSharedResources, 0);
 	}
@@ -721,6 +728,7 @@ struct WindowController::EditImpl : WindowController::Impl
 		showView ();
 
 		window->setContentView (frame);
+
 		return true;
 	}
 
@@ -900,9 +908,10 @@ struct WindowController::EditImpl : WindowController::Impl
 
 	bool canHandleCommand (const Command& command) override
 	{
-		if (command == Commands::Debug::ToggleInlineUIEditor)
+		using namespace Commands;
+		if (command == Debug::ToggleInlineUIEditor || command == Debug::RecreateView)
 			return frame->getModalView () == nullptr;
-		if (command == Commands::Debug::ResaveSharedResources)
+		if (command == Debug::ResaveSharedResources)
 			return true;
 		else if (uiEditController && uiEditController->getMenuController ()->canHandleCommand (
 		                                 command.group, command.name))
@@ -912,12 +921,18 @@ struct WindowController::EditImpl : WindowController::Impl
 
 	bool handleCommand (const Command& command) override
 	{
-		if (command == Commands::Debug::ToggleInlineUIEditor)
+		using namespace Commands;
+		if (command == Debug::ToggleInlineUIEditor)
 		{
 			enableEditing (!isEditing);
 			return true;
 		}
-		else if (command == Commands::Debug::ResaveSharedResources)
+		else if (command == Debug::RecreateView)
+		{
+			showView ();
+			return true;
+		}
+		else if (command == Debug::ResaveSharedResources)
 		{
 			Detail::saveSharedUIDescription ();
 			return true;
@@ -1056,6 +1071,9 @@ const IMenuBuilder* WindowController::getWindowMenuBuilder (const IWindow& windo
 	return nullptr;
 }
 
+ModelBindingPtr WindowController::getModelBinding () const { return impl->modelBinding; }
+CustomizationPtr WindowController::getCustomization () const { return impl->customization; }
+
 //------------------------------------------------------------------------
 } // Anonymous
 
