@@ -14,11 +14,11 @@
 
 #define RAPIDJSON_HAS_STDSTRING 1
 #if DEBUG
-#include "../rapidjson/include/rapidjson/error/en.h"
+#include "../../thirdparty/rapidjson/include/rapidjson/error/en.h"
 #endif
-#include "../rapidjson/include/rapidjson/document.h"
-#include "../rapidjson/include/rapidjson/prettywriter.h"
-#include "../rapidjson/include/rapidjson/reader.h"
+#include "../../thirdparty/rapidjson/include/rapidjson/document.h"
+#include "../../thirdparty/rapidjson/include/rapidjson/prettywriter.h"
+#include "../../thirdparty/rapidjson/include/rapidjson/reader.h"
 
 //------------------------------------------------------------------------
 namespace VSTGUI {
